@@ -16,7 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import codecs
 import copy
 import functools
 import io
@@ -32,6 +31,7 @@ from gi.repository import GObject
 from gi.repository import Gio
 from gi.repository import Gdk
 from gi.repository import Gtk
+from gi.repository import GtkSource
 
 from meld.conf import _
 from . import diffutil
@@ -52,6 +52,16 @@ from .util.compat import text_type
 from meld.sourceview import LanguageManager
 
 
+def with_focused_pane(function):
+    @functools.wraps(function)
+    def wrap_function(*args, **kwargs):
+        pane = args[0]._get_focused_pane()
+        if pane == -1:
+            return
+        return function(args[0], pane, *args[1:], **kwargs)
+    return wrap_function
+
+
 class CachedSequenceMatcher(object):
     """Simple class for caching diff results, with LRU-based eviction
 
@@ -118,30 +128,15 @@ class TaskEntry(object):
             setattr(self, var, val)
 
 
-class TextviewLineAnimation(object):
-    __slots__ = ("start_mark", "end_mark", "start_rgba", "end_rgba",
-                 "start_time", "duration")
-
-    def __init__(self, mark0, mark1, rgba0, rgba1, duration):
-        self.start_mark = mark0
-        self.end_mark = mark1
-        self.start_rgba = rgba0
-        self.end_rgba = rgba1
-        self.start_time = GLib.get_monotonic_time()
-        self.duration = duration
-
-
 class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     """Two or three way comparison of text files"""
 
     __gtype_name__ = "FileDiff"
 
     __gsettings_bindings__ = (
-        ('highlight-current-line', 'highlight-current-line'),
         ('ignore-blank-lines', 'ignore-blank-lines'),
     )
 
-    highlight_current_line = GObject.property(type=bool, default=False)
     ignore_blank_lines = GObject.property(
         type=bool,
         nick="Ignore blank lines",
@@ -161,12 +156,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     # Identifiers for MsgArea messages
     (MSG_SAME, MSG_SLOW_HIGHLIGHT, MSG_SYNCPOINTS) = list(range(3))
 
-    text_windows = {
-        Gtk.TextWindowType.TEXT,
-        Gtk.TextWindowType.LEFT,
-        Gtk.TextWindowType.RIGHT,
-    }
-
     __gsignals__ = {
         'next-conflict-changed': (GObject.SignalFlags.RUN_FIRST, None, (bool, bool)),
         'action-mode-changed': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
@@ -197,16 +186,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             column_sizes.add_widget(widget)
 
         self.warned_bad_comparison = False
-        for v in self.textview:
-            buf = meldbuffer.MeldBuffer()
-            buf.connect('begin_user_action',
-                        self.on_textbuffer_begin_user_action)
-            buf.connect('end_user_action', self.on_textbuffer_end_user_action)
-            v.set_buffer(buf)
-            buf.data.connect('file-changed', self.notify_file_changed)
-            v.late_bind()
         self._keymask = 0
-        self.load_font()
         self.meta = {}
         self.deleted_lines_pending = -1
         self.textview_overwrite = 0
@@ -238,12 +218,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.syncpoints = []
         self.in_nested_textview_gutter_expose = False
         self._cached_match = CachedSequenceMatcher()
-        self.anim_source_id = [None for buf in self.textbuffer]
-        self.animating_chunks = [[] for buf in self.textbuffer]
+
         for buf in self.textbuffer:
-            buf.create_tag("inline")
             buf.connect("notify::has-selection",
                         self.update_text_actions_sensitivity)
+            buf.connect('begin_user_action',
+                        self.on_textbuffer_begin_user_action)
+            buf.connect('end_user_action', self.on_textbuffer_end_user_action)
+            buf.data.connect('file-changed', self.notify_file_changed)
 
         self.ui_file = gnomeglade.ui_file("filediff-ui.xml")
         self.actiongroup = self.FilediffActions
@@ -252,10 +234,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.findbar = findbar.FindBar(self.grid)
         self.grid.attach(self.findbar.widget, 1, 2, 5, 1)
 
-        self.widget.ensure_style()
-        self.on_style_updated(self.widget)
-        self.widget.connect("style-updated", self.on_style_updated)
-
         self.set_num_panes(num_panes)
         self.cursor = CursorDetails()
         self.connect("current-diff-changed", self.on_current_diff_changed)
@@ -278,7 +256,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
         # Prototype implementation
 
-        from meld.gutterrendererchunk import GutterRendererChunkAction
+        from meld.gutterrendererchunk import GutterRendererChunkAction, GutterRendererChunkLines
 
         for pane, t in enumerate(self.textview):
             # FIXME: set_num_panes will break this good
@@ -301,9 +279,24 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 gutter = t.get_gutter(window)
                 gutter.insert(renderer, -40)
 
-        self.connect("notify::ignore-blank-lines", self.refresh_comparison)
+            # TODO: This renderer handling should all be part of
+            # MeldSourceView, but our current diff-chunk-handling makes
+            # this difficult.
+            window = Gtk.TextWindowType.LEFT
+            if direction == Gtk.TextDirection.RTL:
+                window = Gtk.TextWindowType.RIGHT
+            renderer = GutterRendererChunkLines(pane, pane - 1, self.linediffer)
+            renderer.set_properties(
+                "alignment-mode", GtkSource.GutterRendererAlignmentMode.FIRST,
+                "yalign", 0.5,
+                "xalign", 1.0,
+                "xpad", 3,
+            )
+            gutter = t.get_gutter(window)
+            gutter.insert(renderer, -30)
+            t.line_renderer = renderer
 
-        meldsettings.connect('changed', self.on_setting_changed)
+        self.connect("notify::ignore-blank-lines", self.refresh_comparison)
 
     def get_keymask(self):
         return self._keymask
@@ -332,41 +325,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.findbar.start_find_previous(self.focus_pane)
             self.keymask &= ~mod_key
 
-    def on_style_updated(self, widget):
-        style = widget.get_style_context()
-
-        def lookup(name, default):
-            found, colour = style.lookup_color(name)
-            if not found:
-                colour = Gdk.RGBA()
-                colour.parse(default)
-            return colour
-
-        for buf in self.textbuffer:
-            tag = buf.get_tag_table().lookup("inline")
-            tag.props.background_rgba = lookup("inline-bg", "LightSteelBlue2")
-
-        override_bg = style.lookup_color("override-background-color")
-        self.override_bg = override_bg[1] if override_bg[0] else None
-
-        self.fill_colors = {"insert"  : lookup("insert-bg", "DarkSeaGreen1"),
-                            "delete"  : lookup("insert-bg", "DarkSeaGreen1"),
-                            "conflict": lookup("conflict-bg", "Pink"),
-                            "replace" : lookup("replace-bg", "#ddeeff"),
-                            "current-chunk-highlight":
-                                lookup("current-chunk-highlight", '#ffffff')}
-        self.line_colors = {"insert"  : lookup("insert-outline", "#77f077"),
-                            "delete"  : lookup("insert-outline", "#77f077"),
-                            "conflict": lookup("conflict-outline", "#f0768b"),
-                            "replace" : lookup("replace-outline", "#8bbff3")}
-        self.highlight_color = lookup("current-line-highlight", "#ffff00")
-        self.syncpoint_color = lookup("syncpoint-outline", "#555555")
-
-        for associated in self.diffmap + self.linkmap:
-            associated.set_color_scheme([self.fill_colors, self.line_colors])
-
-        self.queue_draw()
-
     def on_focus_change(self):
         self.keymask = 0
 
@@ -657,10 +615,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self._sync_vscroll(self.scrolledwindow[0].get_vadjustment(), 0)
         self.scheduler.add_task(resync)
 
-    def delete_change(self, widget):
-        pane = self._get_focused_pane()
+    @with_focused_pane
+    def delete_change(self, pane):
         chunk = self.linediffer.get_chunk(self.cursor.chunk, pane)
-        assert(pane != -1 and self.cursor.chunk is not None)
+        assert(self.cursor.chunk is not None)
         assert(chunk is not None)
         self.delete_chunk(pane, chunk)
 
@@ -844,20 +802,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._after_text_modified(buffer, starting_at, -self.deleted_lines_pending)
         self.deleted_lines_pending = -1
 
-    def load_font(self):
-        context = self.textview0.get_pango_context()
-        metrics = context.get_metrics(meldsettings.font,
-                                      context.get_language())
-        line_height_points = metrics.get_ascent() + metrics.get_descent()
-        self.pixels_per_line = line_height_points // 1024
-        for i in range(3):
-            self.textview[i].override_font(meldsettings.font)
-        for i in range(2):
-            self.linkmap[i].queue_draw()
-
-    def on_setting_changed(self, settings, key):
-        if key == 'font':
-            self.load_font()
 
     def check_save_modified(self):
         response = Gtk.ResponseType.OK
@@ -950,14 +894,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_undo_checkpointed(self, undosequence, buf, checkpointed):
         self.set_buffer_modified(buf, not checkpointed)
 
-    def open_external(self):
-        pane = self._get_focused_pane()
-        if pane >= 0:
-            if self.textbuffer[pane].data.filename:
-                pos = self.textbuffer[pane].props.cursor_position
-                cursor_it = self.textbuffer[pane].get_iter_at_offset(pos)
-                line = cursor_it.get_line() + 1
-                self._open_files([self.textbuffer[pane].data.filename], line)
+    @with_focused_pane
+    def open_external(self, pane):
+        if not self.textbuffer[pane].data.filename:
+            return
+        pos = self.textbuffer[pane].props.cursor_position
+        cursor_it = self.textbuffer[pane].get_iter_at_offset(pos)
+        line = cursor_it.get_line() + 1
+        self._open_files([self.textbuffer[pane].data.filename], line)
 
     def update_text_actions_sensitivity(self, *args):
         widget = self.focus_pane
@@ -975,15 +919,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     ("Cut", "Copy", "Paste"), (cut, copy, paste)):
                 self.main_actiongroup.get_action(action).set_sensitive(sens)
 
-    def get_selected_text(self):
+    @with_focused_pane
+    def get_selected_text(self, pane):
         """Returns selected text of active pane"""
-        pane = self._get_focused_pane()
-        if pane != -1:
-            buf = self.textbuffer[pane]
-            sel = buf.get_selection_bounds()
-            if sel:
-                return text_type(buf.get_text(sel[0], sel[1], False), 'utf8')
-        return None
+        buf = self.textbuffer[pane]
+        sel = buf.get_selection_bounds()
+        if sel:
+            return text_type(buf.get_text(sel[0], sel[1], False), 'utf8')
 
     def on_find_activate(self, *args):
         selected_text = self.get_selected_text()
@@ -1045,11 +987,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if os.path.exists(buf.data.savefile):
             writable = os.access(buf.data.savefile, os.W_OK)
         self.set_buffer_writable(buf, writable)
-
-        # FIXME: Hack around bgo#737804; remove after GTK+ 3.18 is required
-        def set_merge_file_entry():
-            self.fileentry[1].set_filename(buf.data.savefile)
-        self.scheduler.add_task(set_merge_file_entry)
+        self.fileentry[1].set_filename(buf.data.savefile)
         self.recompute_label()
 
     def _set_save_action_sensitivity(self):
@@ -1112,14 +1050,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._disconnect_buffer_handlers()
         self.linediffer.clear()
         self.queue_draw()
-
-        # Build and uniquify a list of encodings to try out
-        extras = list(settings.get_value('detect-encodings'))
-        builtin = ['utf-8', GLib.get_codeset(), 'utf-16le', 'iso8859-15']
-        allencs = [codecs.lookup(c).name for c in extras + builtin]
-        seen = set()
-        try_codecs = [c for c in allencs if c not in seen and not seen.add(c)]
-
+        try_codecs = list(settings.get_value('detect-encodings'))
+        try_codecs.append('latin1')
         yield _("[%s] Opening files") % self.label_text
         tasks = []
 
@@ -1536,109 +1468,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.text_filters = []
             self.refresh_comparison()
 
-    def on_textview_draw(self, textview, context):
-        if self.num_panes == 1:
-            return
-
-        def should_draw(textwindow):
-            window = textview.get_window(textwindow)
-            if not window:
-                return False
-            return Gtk.cairo_should_draw_window(context, window)
-
-        if not any(should_draw(w) for w in self.text_windows):
-            return
-
-        visible = textview.get_visible_rect()
-        pane = self.textview.index(textview)
-        textbuffer = textview.get_buffer()
-        x, y = textview.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
-                                                0, 0)
-        view_allocation = textview.get_allocation()
-        bounds = (textview.get_line_num_for_y(y),
-                  textview.get_line_num_for_y(y + view_allocation.height + 1))
-
-        width, height = view_allocation.width, view_allocation.height
-        context.set_line_width(1.0)
-
-        if self.override_bg:
-            context.set_source_rgba(*self.override_bg)
-            context.rectangle(0, 0, width, height)
-            context.fill()
-
-        for change in self.linediffer.single_changes(pane, bounds):
-            ypos0 = textview.get_y_for_line_num(change[1]) - visible.y
-            ypos1 = textview.get_y_for_line_num(change[2]) - visible.y
-
-            context.rectangle(-0.5, ypos0 - 0.5, width + 1, ypos1 - ypos0)
-            if change[1] != change[2]:
-                context.set_source_rgba(*self.fill_colors[change[0]])
-                context.fill_preserve()
-                if self.linediffer.locate_chunk(pane, change[1])[0] == self.cursor.chunk:
-                    highlight = self.fill_colors['current-chunk-highlight']
-                    context.set_source_rgba(*highlight)
-                    context.fill_preserve()
-
-            context.set_source_rgba(*self.line_colors[change[0]])
-            context.stroke()
-
-        if (self.props.highlight_current_line and textview.is_focus() and
-                self.cursor.line is not None):
-            it = textbuffer.get_iter_at_line(self.cursor.line)
-            ypos, line_height = textview.get_line_yrange(it)
-            context.save()
-            context.rectangle(0, ypos - visible.y, width, line_height)
-            context.clip()
-            context.set_source_rgba(*self.highlight_color)
-            context.paint_with_alpha(0.25)
-            context.restore()
-
-        for syncpoint in [p[pane] for p in self.syncpoints]:
-            if not syncpoint:
-                continue
-            syncline = textbuffer.get_iter_at_mark(syncpoint).get_line()
-            if bounds[0] <= syncline <= bounds[1]:
-                ypos = textview.get_y_for_line_num(syncline) - visible.y
-                context.rectangle(-0.5, ypos - 0.5, width + 1, 1)
-                context.set_source_rgba(*self.syncpoint_color)
-                context.stroke()
-
-        new_anim_chunks = []
-        for c in self.animating_chunks[pane]:
-            current_time = GLib.get_monotonic_time()
-            percent = min(1.0, (current_time - c.start_time) / float(c.duration))
-            rgba_pairs = zip(c.start_rgba, c.end_rgba)
-            rgba = [s + (e - s) * percent for s, e in rgba_pairs]
-
-            it = textbuffer.get_iter_at_mark(c.start_mark)
-            ystart, _ = textview.get_line_yrange(it)
-            it = textbuffer.get_iter_at_mark(c.end_mark)
-            yend, _ = textview.get_line_yrange(it)
-            if ystart == yend:
-                ystart -= 1
-
-            context.set_source_rgba(*rgba)
-            context.rectangle(0, ystart - visible.y, width, yend - ystart)
-            context.fill()
-
-            if current_time <= c.start_time + c.duration:
-                new_anim_chunks.append(c)
-            else:
-                textbuffer.delete_mark(c.start_mark)
-                textbuffer.delete_mark(c.end_mark)
-        self.animating_chunks[pane] = new_anim_chunks
-
-        if self.animating_chunks[pane] and self.anim_source_id[pane] is None:
-            def anim_cb():
-                textview.queue_draw()
-                return True
-            # Using timeout_add interferes with recalculation of inline
-            # highlighting; this mechanism could be improved.
-            self.anim_source_id[pane] = GLib.idle_add(anim_cb)
-        elif not self.animating_chunks[pane] and self.anim_source_id[pane]:
-            GLib.source_remove(self.anim_source_id[pane])
-            self.anim_source_id[pane] = None
-
     def _get_filename_for_saving(self, title ):
         dialog = Gtk.FileChooserDialog(title,
             parent=self.widget.get_toplevel(),
@@ -1698,15 +1527,15 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             else:
                 prompt = _("Save Right Pane As")
             filename = self._get_filename_for_saving(prompt)
-            if filename:
-                bufdata.filename = bufdata.label = os.path.abspath(filename)
-                bufdata.savefile = None
-                self.fileentry[pane].set_filename(bufdata.filename)
-                self.filelabel_toolitem[pane].set_visible(False)
-                self.fileentry_toolitem[pane].set_visible(True)
-            else:
+            if not filename:
                 return False
 
+            bufdata.filename = bufdata.label = os.path.abspath(filename)
+            bufdata.savefile = None
+            self.fileentry[pane].set_filename(bufdata.filename)
+            self.filelabel_toolitem[pane].set_visible(False)
+            self.fileentry_toolitem[pane].set_visible(True)
+
         if not force_overwrite and not bufdata.current_on_disk():
             gfile = Gio.File.new_for_path(bufdata.filename)
             primary = (
@@ -1817,15 +1646,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         for linkmap in self.linkmap:
             linkmap.queue_draw()
 
-    def save(self):
-        pane = self._get_focused_pane()
-        if pane >= 0:
-            self.save_file(pane)
+    @with_focused_pane
+    def save(self, pane):
+        self.save_file(pane)
 
-    def save_as(self):
-        pane = self._get_focused_pane()
-        if pane >= 0:
-            self.save_file(pane, True)
+    @with_focused_pane
+    def save_as(self, pane):
+        self.save_file(pane, saveas=True)
 
     def on_save_all_activate(self, action):
         for i in range(self.num_panes):
@@ -1977,6 +1804,22 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
             self.actiongroup.get_action("MakePatch").set_sensitive(n > 1)
 
+            def chunk_iter(i):
+                def chunks(bounds):
+                    for chunk in self.linediffer.single_changes(i, bounds):
+                        yield chunk
+                return chunks
+
+            def current_chunk_check(i):
+                def chunks(change):
+                    chunk = self.linediffer.locate_chunk(i, change[1])[0]
+                    return chunk == self.cursor.chunk
+                return chunks
+
+            for (w, i) in zip(self.textview, range(self.num_panes)):
+                w.chunk_iter = chunk_iter(i)
+                w.current_chunk_check = current_chunk_check(i)
+
             def coords_iter(i):
                 buf_index = 2 if i == 1 and self.num_panes == 3 else i
                 get_end_iter = self.textbuffer[buf_index].get_end_iter
@@ -1997,7 +1840,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
             for (w, i) in zip(self.diffmap, (0, self.num_panes - 1)):
                 scroll = self.scrolledwindow[i].get_vscrollbar()
-                w.setup(scroll, coords_iter(i), [self.fill_colors, self.line_colors])
+                w.setup(scroll, coords_iter(i))
 
             for (w, i) in zip(self.linkmap, (0, self.num_panes - 2)):
                 w.associate(self, self.textview[i], self.textview[i + 1])
@@ -2023,7 +1866,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             dst_start = b1.get_iter_at_line_or_eof(chunk[3])
             mark0 = b1.create_mark(None, dst_start, True)
             new_end = b1.insert_at_line(chunk[3], t0)
-        else: # copy down
+        else:
             dst_start = b1.get_iter_at_line_or_eof(chunk[4])
             mark0 = b1.create_mark(None, dst_start, True)
             new_end = b1.insert_at_line(chunk[4], t0)
@@ -2031,12 +1874,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         mark1 = b1.create_mark(None, new_end, True)
         # FIXME: If the inserted chunk ends up being an insert chunk, then
         # this animation is not visible; this happens often in three-way diffs
-        rgba0 = self.fill_colors['insert'].copy()
-        rgba1 = self.fill_colors['insert'].copy()
-        rgba0.alpha = 1.0
-        rgba1.alpha = 0.0
-        anim = TextviewLineAnimation(mark0, mark1, rgba0, rgba1, 500000)
-        self.animating_chunks[dst].append(anim)
+        self.textview[dst].add_fading_highlight(mark0, mark1, 'insert', 500000)
 
     def replace_chunk(self, src, dst, chunk):
         b0, b1 = self.textbuffer[src], self.textbuffer[dst]
@@ -2053,18 +1891,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         mark1 = b1.create_mark(None, new_end, True)
         if chunk[1] == chunk[2]:
             # TODO: Need a more specific colour here; conflict is wrong
-            rgba0 = self.fill_colors['conflict'].copy()
-            rgba1 = self.fill_colors['conflict'].copy()
+            colour = 'conflict'
         else:
             # FIXME: If the inserted chunk ends up being an insert chunk, then
             # this animation is not visible; this happens often in three-way
             # diffs
-            rgba0 = self.fill_colors['insert'].copy()
-            rgba1 = self.fill_colors['insert'].copy()
-        rgba0.alpha = 1.0
-        rgba1.alpha = 0.0
-        anim = TextviewLineAnimation(mark0, mark1, rgba0, rgba1, 500000)
-        self.animating_chunks[dst].append(anim)
+            colour = 'insert'
+        self.textview[dst].add_fading_highlight(mark0, mark1, colour, 500000)
 
     def delete_chunk(self, src, chunk):
         b0 = self.textbuffer[src]
@@ -2075,18 +1908,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         mark0 = b0.create_mark(None, it, True)
         mark1 = b0.create_mark(None, it, True)
         # TODO: Need a more specific colour here; conflict is wrong
-        rgba0 = self.fill_colors['conflict'].copy()
-        rgba1 = self.fill_colors['conflict'].copy()
-        rgba0.alpha = 1.0
-        rgba1.alpha = 0.0
-        anim = TextviewLineAnimation(mark0, mark1, rgba0, rgba1, 500000)
-        self.animating_chunks[src].append(anim)
-
-    def add_sync_point(self, action):
-        pane = self._get_focused_pane()
-        if pane == -1:
-            return
+        self.textview[src].add_fading_highlight(
+            mark0, mark1, 'conflict', 500000)
 
+    @with_focused_pane
+    def add_sync_point(self, pane, action):
         # Find a non-complete syncpoint, or create a new one
         if self.syncpoints and None in self.syncpoints[-1]:
             syncpoint = self.syncpoints.pop()
@@ -2097,6 +1923,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         syncpoint[pane] = self.textbuffer[pane].create_mark(None, cursor_it)
         self.syncpoints.append(syncpoint)
 
+        for i, t in enumerate(self.textview[:self.num_panes]):
+            t.syncpoints = [p[i] for p in self.syncpoints if p[i] is not None]
+
         def make_line_retriever(pane, marks):
             buf = self.textbuffer[pane]
             mark = marks[pane]
@@ -2135,6 +1964,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def clear_sync_points(self, action):
         self.syncpoints = []
         self.linediffer.syncpoints = []
+        for t in enumerate(self.textview):
+            t.syncpoints = []
         for mgr in self.msgarea_mgr:
             if mgr.get_msg_id() == FileDiff.MSG_SYNCPOINTS:
                 mgr.clear()
