@@ -19,13 +19,21 @@ import math
 
 from gi.repository import Gtk
 
+from meld.misc import get_common_theme
+from meld.settings import meldsettings
+
+
+# Rounded rectangle corner radius for culled changes display
+RADIUS = 3
+
 
 class LinkMap(Gtk.DrawingArea):
 
     __gtype_name__ = "LinkMap"
 
     def __init__(self):
-        self._setup = False
+        self.filediff = None
+        meldsettings.connect('changed', self.on_setting_changed)
 
     def associate(self, filediff, left_view, right_view):
         self.filediff = filediff
@@ -34,30 +42,25 @@ class LinkMap(Gtk.DrawingArea):
             self.views.reverse()
         self.view_indices = [filediff.textview.index(t) for t in self.views]
 
-        self.set_color_scheme((filediff.fill_colors, filediff.line_colors))
+        self.on_setting_changed(meldsettings, 'style-scheme')
 
-        self.line_height = filediff.pixels_per_line
-
-        self._setup = True
-
-    def set_color_scheme(self, color_map):
-        self.fill_colors, self.line_colors = color_map
-        self.queue_draw()
+    def on_setting_changed(self, settings, key):
+        if key == 'style-scheme':
+            self.fill_colors, self.line_colors = get_common_theme()
 
     def do_draw(self, context):
-        if not self._setup:
+        if not self.filediff:
             return
 
         context.set_line_width(1.0)
         allocation = self.get_allocation()
-        style = self.get_style_context()
 
         pix_start = [t.get_visible_rect().y for t in self.views]
-        y_offset = [t.translate_coordinates(self, 0, 0)[1] for t in self.views]
+        y_offset = [t.translate_coordinates(self, 0, 0)[1] + 1 for t in self.views]
 
+        clip_y = min(y_offset) - 1
         clip_height = max(t.get_visible_rect().height for t in self.views) + 2
-        Gtk.render_frame(style, context, 0, 0, allocation.width, clip_height)
-        context.rectangle(0, -1, allocation.width, clip_height)
+        context.rectangle(0, clip_y, allocation.width, clip_height)
         context.clip()
 
         height = allocation.height
@@ -69,8 +72,6 @@ class LinkMap(Gtk.DrawingArea):
         wtotal = allocation.width
         # For bezier control points
         x_steps = [-0.5, (1. / 3) * wtotal, (2. / 3) * wtotal, wtotal + 0.5]
-        # Rounded rectangle corner radius for culled changes display
-        radius = self.line_height // 3
         q_rad = math.pi / 2
 
         left, right = self.view_indices
@@ -80,20 +81,23 @@ class LinkMap(Gtk.DrawingArea):
             # f and t are short for "from" and "to"
             f0, f1 = [view_offset_line(0, l) for l in c[1:3]]
             t0, t1 = [view_offset_line(1, l) for l in c[3:5]]
+            # We want the last pixel of the previous line
+            f1 = f1 if f1 == f0 else f1 - 1
+            t1 = t1 if t1 == t0 else t1 - 1
 
             # If either endpoint is completely off-screen, we cull for clarity
             if (t0 < 0 and t1 < 0) or (t0 > height and t1 > height):
                 if f0 == f1:
                     continue
-                context.arc(x_steps[0], f0 - 0.5 + radius, radius, -q_rad, 0)
-                context.arc(x_steps[0], f1 - 0.5 - radius, radius, 0, q_rad)
+                context.arc(x_steps[0], f0 - 0.5 + RADIUS, RADIUS, -q_rad, 0)
+                context.arc(x_steps[0], f1 - 0.5 - RADIUS, RADIUS, 0, q_rad)
                 context.close_path()
             elif (f0 < 0 and f1 < 0) or (f0 > height and f1 > height):
                 if t0 == t1:
                     continue
-                context.arc_negative(x_steps[3], t0 - 0.5 + radius, radius,
+                context.arc_negative(x_steps[3], t0 - 0.5 + RADIUS, RADIUS,
                                      -q_rad, q_rad * 2)
-                context.arc_negative(x_steps[3], t1 - 0.5 - radius, radius,
+                context.arc_negative(x_steps[3], t1 - 0.5 - RADIUS, RADIUS,
                                      q_rad * 2, q_rad)
                 context.close_path()
             else:
@@ -112,9 +116,8 @@ class LinkMap(Gtk.DrawingArea):
 
             chunk_idx = self.filediff.linediffer.locate_chunk(left, c[1])[0]
             if chunk_idx == self.filediff.cursor.chunk:
-                h = self.fill_colors['current-chunk-highlight']
-                context.set_source_rgba(
-                    h.red, h.green, h.blue, 0.5)
+                highlight = self.fill_colors['current-chunk-highlight']
+                context.set_source_rgba(*highlight)
                 context.fill_preserve()
 
             context.set_source_rgba(*self.line_colors[c[0]])
@@ -122,3 +125,19 @@ class LinkMap(Gtk.DrawingArea):
 
     def do_scroll_event(self, event):
         self.filediff.next_diff(event.direction)
+
+
+class ScrollLinkMap(Gtk.DrawingArea):
+
+    __gtype_name__ = "ScrollLinkMap"
+
+    def __init__(self):
+        self.melddoc = None
+
+    def associate(self, melddoc):
+        self.melddoc = melddoc
+
+    def do_scroll_event(self, event):
+        if not self.melddoc:
+            return
+        self.melddoc.next_diff(event.direction)
