@@ -24,7 +24,10 @@ import shutil
 import re
 import subprocess
 
+from gi.repository import Gdk
+from gi.repository import GObject
 from gi.repository import Gtk
+from gi.repository import GtkSource
 
 from meld.conf import _
 
@@ -141,10 +144,84 @@ def make_tool_button_widget(label):
     return hbox
 
 
+MELD_STYLE_SCHEME = "meld-base"
+
+
+def colour_lookup_with_fallback(name, attribute):
+    from meld.settings import meldsettings
+    source_style = meldsettings.style_scheme
+
+    style = source_style.get_style(name)
+    style_attr = getattr(style.props, attribute) if style else None
+    if not style or not style_attr:
+        manager = GtkSource.StyleSchemeManager.get_default()
+        source_style = manager.get_scheme(MELD_STYLE_SCHEME)
+        try:
+            style = source_style.get_style(name)
+            style_attr = getattr(style.props, attribute)
+        except AttributeError:
+            pass
+
+    if not style_attr:
+        import sys
+        print >> sys.stderr, _(
+            "Couldn't find colour scheme details for %s-%s; "
+            "this is a bad install") % (name, attribute)
+        sys.exit(1)
+
+    colour = Gdk.RGBA()
+    colour.parse(style_attr)
+    return colour
+
+
+def get_common_theme():
+    lookup = colour_lookup_with_fallback
+    fill_colours = {
+        "insert": lookup("meld:insert", "background"),
+        "delete": lookup("meld:insert", "background"),
+        "conflict": lookup("meld:conflict", "background"),
+        "replace": lookup("meld:replace", "background"),
+        "current-chunk-highlight": lookup(
+            "meld:current-chunk-highlight", "background")
+    }
+    line_colours = {
+        "insert": lookup("meld:insert", "line-background"),
+        "delete": lookup("meld:insert", "line-background"),
+        "conflict": lookup("meld:conflict", "line-background"),
+        "replace": lookup("meld:replace", "line-background"),
+    }
+    return fill_colours, line_colours
+
+
 def gdk_to_cairo_color(color):
     return (color.red / 65535., color.green / 65535., color.blue / 65535.)
 
 
+def fallback_decode(bytes, encodings, lossy=False):
+    """Try and decode bytes according to multiple encodings
+
+    Generally, this should be used for best-effort decoding, when the
+    desired behaviour is "probably this, or UTF-8".
+
+    If lossy is True, then decode errors will be replaced. This may be
+    reasonable when the string is for display only.
+    """
+    if isinstance(bytes, unicode):
+        return bytes
+
+    for encoding in encodings:
+        try:
+            return bytes.decode(encoding)
+        except UnicodeDecodeError:
+            pass
+
+    if lossy:
+        return bytes.decode(encoding, errors='replace')
+
+    raise ValueError(
+        "Couldn't decode %r as one of %r" % (bytes, encodings))
+
+
 def all_same(lst):
     """Return True if all elements of the list are equal"""
     return not lst or lst.count(lst[0]) == len(lst)
@@ -178,13 +255,12 @@ def shorten_names(*names):
     return [name or _("[None]") for name in basenames]
 
 
-def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
+def read_pipe_iter(command, workdir, errorstream, yield_interval=0.1):
     """Read the output of a shell command iteratively.
 
     Each time 'callback_interval' seconds pass without reading any data,
     this function yields None.
     When all the data is read, the entire string is yielded.
-    If 'workdir' is specified the command is run from that directory.
     """
     class sentinel(object):
         def __init__(self):
@@ -232,10 +308,7 @@ def read_pipe_iter(command, errorstream, yield_interval=0.1, workdir=None):
             self.proc = None
             if status:
                 errorstream.error("Exit code: %i\n" % status)
-            yield "".join(bits)
-            yield status
-    if workdir == "":
-        workdir = None
+            yield status, "".join(bits)
     return sentinel()()
 
 
@@ -248,23 +321,6 @@ def write_pipe(command, text, error=None):
     return proc.wait()
 
 
-def commonprefix(dirs):
-    """Given a list of pathnames, returns the longest common leading component.
-    """
-    if not dirs:
-        return ''
-    n = [d.split(os.sep) for d in dirs]
-    prefix = n[0]
-    for item in n:
-        for i in range(len(prefix)):
-            if prefix[:i+1] != item[:i+1]:
-                prefix = prefix[:i]
-                if i == 0:
-                    return ''
-                break
-    return os.sep.join(prefix)
-
-
 def copy2(src, dst):
     """Like shutil.copy2 but ignores chmod errors, and copies symlinks as links
     See [Bug 568000] Copying to NTFS fails
