@@ -1,27 +1,29 @@
-# coding=utf-8
-### Copyright (C) 2002-2005 Stephen Kennedy <stevek@gnome.org>
-### Copyright (C) 2005 Aaron Bentley <aaron.bentley@utoronto.ca>
+# -*- coding: utf-8 -*-
+# Copyright (C) 2002-2005 Stephen Kennedy <stevek@gnome.org>
+# Copyright (C) 2005 Aaron Bentley <aaron.bentley@utoronto.ca>
+# Copyright (C) 2015 Kai Willadsen <kai.willadsen@gmail.com>
 
-### Redistribution and use in source and binary forms, with or without
-### modification, are permitted provided that the following conditions
-### are met:
-### 
-### 1. Redistributions of source code must retain the above copyright
-###    notice, this list of conditions and the following disclaimer.
-### 2. Redistributions in binary form must reproduce the above copyright
-###    notice, this list of conditions and the following disclaimer in the
-###    documentation and/or other materials provided with the distribution.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-### THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
-### IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-### OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-### IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
-### INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-### NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-### DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-### THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
-### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from collections import defaultdict
 
 import errno
@@ -34,7 +36,7 @@ import tempfile
 from . import _vc
 
 
-class Vc(_vc.CachedVc):
+class Vc(_vc.Vc):
 
     CMD = "bzr"
     CMDARGS = ["--no-aliases", "--no-plugins"]
@@ -49,8 +51,6 @@ class Vc(_vc.CachedVc):
         _vc.STATE_MODIFIED, _vc.STATE_RENAMED, _vc.STATE_NEW, _vc.STATE_REMOVED
     )
 
-    VC_COLUMNS = (_vc.DATA_NAME, _vc.DATA_STATE, _vc.DATA_OPTIONS)
-
     conflict_map = {
         _vc.CONFLICT_BASE: '.BASE',
         _vc.CONFLICT_OTHER: '.OTHER',
@@ -89,16 +89,15 @@ class Vc(_vc.CachedVc):
                                             ''.join(state_2_map.keys()),
                                             ''.join(state_3_map.keys()),)
 
-    def __init__(self, location):
-        super(Vc, self).__init__(location)
-        self._tree_cache = {}
-        self._tree_meta_cache = {}
+    def add(self, runner, files):
+        fullcmd = [self.CMD] + self.CMDARGS
+        command = [fullcmd, 'add']
+        runner(command, files, refresh=True, working_dir=self.root)
 
-    def commit_command(self, message):
-        return [self.CMD] + self.CMDARGS + ["commit", "-m", message]
-
-    def add_command(self):
-        return [self.CMD] + self.CMDARGS + ["add"]
+    def commit(self, runner, files, message):
+        fullcmd = [self.CMD] + self.CMDARGS
+        command = [fullcmd, 'commit', '-m', message]
+        runner(command, [], refresh=True, working_dir=self.root)
 
     def revert(self, runner, files):
         runner(
@@ -110,7 +109,7 @@ class Vc(_vc.CachedVc):
             [self.CMD] + self.CMDARGS + ["push"], [], refresh=True,
             working_dir=self.root)
 
-    def update(self, runner, files):
+    def update(self, runner):
         # TODO: Handle checkouts/bound branches by calling
         # update instead of pull. For now we've replicated existing
         # functionality, as update will not work for unbound branches.
@@ -132,24 +131,23 @@ class Vc(_vc.CachedVc):
     def valid_repo(cls, path):
         return not _vc.call([cls.CMD, "root"], cwd=path)
 
-    def get_working_directory(self, workdir):
-        return self.root
-
     def get_files_to_commit(self, paths):
         files = []
         for p in paths:
             if os.path.isdir(p):
-                entries = self._lookup_tree_cache(p)
-                names = [
-                    x for x, y in entries.items() if y in self.commit_statuses]
-                files.extend(names)
+                for path, status in self._tree_cache.items():
+                    if status in self.commit_statuses and path.startswith(p):
+                        files.append(os.path.relpath(path, self.root))
             else:
                 files.append(os.path.relpath(p, self.root))
         return sorted(list(set(files)))
 
-    def _lookup_tree_cache(self, rootdir):
+    def _update_tree_state_cache(self, path):
+        # FIXME: This actually clears out state information, because the
+        # current API doesn't have any state outside of _tree_cache.
         branch_root = _vc.popen(
-            [self.CMD] + self.CMDARGS + ["root", rootdir]).read().rstrip('\n')
+            [self.CMD] + self.CMDARGS + ["root", path],
+            cwd=self.location).read().rstrip('\n')
         entries = []
         while 1:
             try:
@@ -161,8 +159,8 @@ class Vc(_vc.CachedVc):
                 if e.errno != errno.EAGAIN:
                     raise
 
-        self._tree_cache = tree_cache = defaultdict(set)
-        self._tree_meta_cache = tree_meta_cache = defaultdict(list)
+        tree_cache = defaultdict(set)
+        tree_meta_cache = defaultdict(list)
         self._rename_cache = rename_cache = {}
         self._reverse_rename_cache = {}
         # Files can appear twice in the list if they conflict and were renamed
@@ -204,8 +202,7 @@ class Vc(_vc.CachedVc):
                 if executable_match:
                     meta.append(executable_match.group(2))
 
-
-
+            path = path[:-1] if path.endswith('/') else path
             tree_cache[path].update(states)
             tree_meta_cache[path].extend(meta)
 
@@ -218,38 +215,9 @@ class Vc(_vc.CachedVc):
                 del tree_meta_cache[old]
             self._reverse_rename_cache[new] = old
 
-        tree_cache = dict((x, max(y)) for x,y in tree_cache.items())
-        return tree_cache
-
-    def _get_dirsandfiles(self, directory, dirs, files):
-        tree = self._get_tree_cache(directory)
-
-        retfiles = []
-        retdirs = []
-        bzrfiles = {}
-        for path, state in tree.items():
-            mydir, name = os.path.split(path)
-            if path.endswith('/'):
-                mydir, name = os.path.split(mydir)
-            if mydir != directory:
-                continue
-            meta = ','.join(self._tree_meta_cache.get(path, []))
-            if path.endswith('/'):
-                retdirs.append(_vc.Dir(path[:-1], name, state, options=meta))
-            else:
-                retfiles.append(_vc.File(path, name, state, options=meta))
-            bzrfiles[name] = 1
-        for f, path in files:
-            if f not in bzrfiles:
-                state = _vc.STATE_NORMAL
-                meta = ','.join(self._tree_meta_cache.get(path, []))
-                retfiles.append(_vc.File(path, f, state, options=meta))
-        for d, path in dirs:
-            if d not in bzrfiles:
-                state = _vc.STATE_NORMAL
-                meta = ','.join(self._tree_meta_cache.get(path, []))
-                retdirs.append(_vc.Dir(path, d, state, options=meta))
-        return retdirs, retfiles
+        self._tree_cache.update(
+            dict((x, max(y)) for x, y in tree_cache.items()))
+        self._tree_meta_cache = dict(tree_meta_cache)
 
     def get_path_for_repo_file(self, path, commit=None):
         if not path.startswith(self.root + os.path.sep):
@@ -282,30 +250,3 @@ class Vc(_vc.CachedVc):
 
         # bzr paths are all temporary files
         return "%s%s" % (path, self.conflict_map[conflict]), False
-
-    # Sensitivity button mappings.
-    def update_actions_for_paths(self, path_states, actions):
-        states = path_states.values()
-
-        actions["VcCompare"] = bool(path_states)
-        # TODO: We can't disable this for NORMAL, because folders don't
-        # inherit any state from their children, but committing a folder with
-        # modified children is expected behaviour.
-        actions["VcCommit"] = all(s not in (
-            _vc.STATE_NONE, _vc.STATE_IGNORED) for s in states)
-
-        actions["VcUpdate"] = True
-        # TODO: We can't do this; this shells out for each selection change...
-        # actions["VcPush"] = bool(self.get_commits_to_push())
-        actions["VcPush"] = True
-
-        actions["VcAdd"] = all(s not in (
-            _vc.STATE_NORMAL, _vc.STATE_REMOVED) for s in states)
-        actions["VcResolved"] = all(s == _vc.STATE_CONFLICT for s in states)
-        actions["VcRemove"] = (all(s not in (
-            _vc.STATE_NONE, _vc.STATE_IGNORED,
-            _vc.STATE_REMOVED) for s in states) and
-            self.root not in path_states.keys())
-        actions["VcRevert"] = all(s not in (
-            _vc.STATE_NONE, _vc.STATE_NORMAL,
-            _vc.STATE_IGNORED) for s in states)
